#!/usr/bin/python
# rtmpget is part of Loose Seal.  Copyright (C) 2006 Kevin Vance
# <kvance@kvance.com>.
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# 
# -----------------------------------------------------------------------------
#
# This is a proof of concept to demonstrate the retrieval of Flash a/v streams
# over RTMP.  You will need to update the stream_path and remote_host variables
# with valid information in order to start.  If successful, raw audio and video
# will be dumped into the files 'audio.out' and 'video.out' in the current
# directory.  Important data about the stream, including frame size and codec
# IDs will be dumped into 'metadata.out' in the default pickle format.  The
# program will NOT terminate automatically when the stream is finished.  
#
# The audio stream is MP3 and should be playable with XMMS.
#
# The video stream is hopefully of type VP6F, and after being placed in a
# container should be playable with ffmpeg.


# remote_host is the hostname or IP address of the target.
remote_host = 'example.com'

# stream_path should be the location of the stream on the server.
stream_path = 'dir1/dir2/file'


if remote_host == 'example.com':
    print "This program must be manually configured.  Examine rtmpget.py."
    sys.exit()


import pickle
import rtmp
import threading
import time
import sys


# This block of information describes the client and our target.
# ondemand seems to be the app for streaming video.
# tcUrl is the RTMP URL to the app.
# pageURL is the URL to the referring web page.
# swfUrl is the URL to the referring swf file.
flash_info = {
    'app'        : 'ondemand',
    'flashVer'   : 'WIN 8,0,22,0',
    'swfUrl'     : 'http://www.example.com/referring.swf',
    'tcUrl'      : 'rtmp://'+remote_host+':1935/ondemand',
    'fpad'       : False,
    'audioCodecs': 615,
    'videoCodecs': 76,
    'pageUrl'    : 'http://www.example.com/referring.html',
}

# A little logic to do the bandwidth check.
class BWChecker:
    def __init__(self):
        self.count = 1
        self.doneEvent = threading.Event()

    def onBWCheck(self, *args):
        retval = self.count
        self.count += 1
        return (None, retval)

    def onBWDone(self, *args):
        self.doneEvent.set()
        return None

    def wait_until_done(self, timeout=None):
        self.doneEvent.wait(timeout)

# Metadata handler.  Just save it to disk.
def onMetaData(metadata):
    md_file = file('metadata.out', 'wt')
    pickle.dump(metadata, md_file)
    md_file.close()


bwchecker = BWChecker()
conn = rtmp.RTMPConnection(remote_host)
conn.add_client_function('onBWCheck', bwchecker.onBWCheck)
conn.add_client_function('onBWDone' , bwchecker.onBWDone)
conn.set_metadata_function(onMetaData)
conn.connect()

print "> connect"
print conn.call('connect', flash_info, True)

print "> wait for bandwidth check"
bwchecker.wait_until_done()
print "finished bandwidth check"

print "> getStreamLength"
print conn.call('getStreamLength', None, stream_path)

print "> createStream"
stream_id = conn.call('createStream', None)
print stream_id

print "> play"
conn.call('play', None, stream_path, 0, -1000, channel=8, stream=stream_id, \
          no_retval=True)

print "> set buffer time"
conn.set_buffer_time(5000) # Set a high buffering time; this isn't realtime.


audfile = file('audio.out', 'wb')
vidfile = file('video.out', 'wb')
try:
    while True:
        a_data = conn.read_audio()
        v_data = conn.read_video()
        if len(a_data) != 0:
            print "a",
            audfile.write(a_data)
        if len(v_data) != 0:
            print "v",
            vidfile.write(v_data)
        sys.stdout.flush()
        time.sleep(0.5)
finally:
    vidfile.close()
    audfile.close()
    conn.close()
